@@ -33,15 +33,12 @@ package com.thoughtworks.paranamer;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.File;
+import java.lang.reflect.AccessibleObject;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
-import java.lang.reflect.AccessibleObject;
 import java.lang.reflect.Modifier;
-import java.util.ArrayList;
-import java.util.List;
-import java.util.Map;
 import java.util.HashMap;
+import java.util.Map;
 
 
 /**
@@ -55,7 +52,6 @@ import java.util.HashMap;
  */
 public class BytecodeReadingParanamer implements Paranamer {
 
-    private static final String[] EMPTY_NAMES = new String[]{};
     private static final Map primitives = new HashMap() {
         {
             put("int","I");
@@ -68,7 +64,11 @@ public class BytecodeReadingParanamer implements Paranamer {
         }
     };
 
-    public String[] lookupParameterNames(AccessibleObject methodOrCtor) {
+    public String[] lookupParameterNames(AccessibleObject methodOrConstructor) {
+        return lookupParameterNames(methodOrConstructor, true);
+    }
+
+    public String[] lookupParameterNames(AccessibleObject methodOrCtor, boolean throwExceptionIfMissing) {
 
         Class[] types = null;
         Class declaringClass = null;
@@ -85,46 +85,28 @@ public class BytecodeReadingParanamer implements Paranamer {
             name = "<init>";
         }
 
+        if (types.length == 0) {
+            return EMPTY_NAMES;
+        }
         InputStream content = getClassAsStream(declaringClass);
         if (content == null) {
-            throw new ParameterNamesNotFoundException("Unable to get class bytes");
+            if (throwExceptionIfMissing) {
+                throw new ParameterNamesNotFoundException("Unable to get class bytes");
+            } else {
+                return Paranamer.EMPTY_NAMES;
+            }
         }
         try {
             ClassReader reader = new ClassReader(content);
-            TypeCollector visitor = new TypeCollector(name, types);
+            TypeCollector visitor = new TypeCollector(name, types, throwExceptionIfMissing);
             reader.accept(visitor);
             return visitor.getParameterNamesForMethod();
         } catch (IOException e) {
-            throw new ParameterNamesNotFoundException("IoException while reading class bytes", e);
-        }
-    }
-
-    public int areParameterNamesAvailable(Class clazz, String constructorOrMethodName) {
-        InputStream content = getClassAsStream(clazz.getClassLoader(), clazz.getName());
-        if (content == null) {
-            return NO_PARAMETER_NAMES_FOR_CLASS;
-        }
-        try {
-            ClassReader reader = new ClassReader(content);
-            //TODO - also for constructors
-            List methods = getMatchingMethods(clazz.getClassLoader(), clazz.getName(), constructorOrMethodName);
-            if (methods.size() == 0) {
-                return Paranamer.NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-            }
-            TypeCollector visitor = new TypeCollector(constructorOrMethodName, ((Method) methods.get(0)).getParameterTypes());
-            reader.accept(visitor);
-            if (visitor.isClassFound()) {
-                if (!visitor.isMethodFound()) {
-                    return Paranamer.NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-                }
+            if (throwExceptionIfMissing) {
+                throw new ParameterNamesNotFoundException("IoException while reading class bytes", e);
             } else {
-                return Paranamer.NO_PARAMETER_NAMES_FOR_CLASS;
+                return Paranamer.EMPTY_NAMES;
             }
-            return Paranamer.PARAMETER_NAMES_FOUND;
-        } catch (IOException e) {
-            return Paranamer.NO_PARAMETER_NAMES_FOR_CLASS;
-        } catch (ClassNotFoundException e) {
-            return NO_PARAMETER_NAMES_FOR_CLASS;
         }
     }
 
@@ -147,18 +129,6 @@ public class BytecodeReadingParanamer implements Paranamer {
         return asStream;
     }
 
-    private List getMatchingMethods(ClassLoader classLoader, String className, String name) throws ClassNotFoundException {
-        List list = new ArrayList();
-        Method[] methods = classLoader.loadClass(className).getMethods();
-        for (int i = 0; i < methods.length; i++) {
-            Method method = methods[i];
-            if (method.getName().equals(name)) {
-                list.add(method);
-            }
-        }
-        return list;
-    }
-
     /**
      * The type collector waits for an specific method in order to start a method
      * collector.
@@ -172,14 +142,16 @@ public class BytecodeReadingParanamer implements Paranamer {
         private final String methodName;
 
         private final Class[] parameterTypes;
+        private final boolean throwExceptionIfMissing;
 
         private MethodCollector collector;
         private boolean methodFound = false;
         private boolean classFound = false;
 
-        private TypeCollector(String methodName, Class[] parameterTypes) {
+        private TypeCollector(String methodName, Class[] parameterTypes, boolean throwExceptionIfMissing) {
             this.methodName = methodName;
             this.parameterTypes = parameterTypes;
+            this.throwExceptionIfMissing = throwExceptionIfMissing;
             this.collector = null;
         }
 
@@ -235,21 +207,18 @@ public class BytecodeReadingParanamer implements Paranamer {
 
         private String[] getParameterNamesForMethod() {
             if (collector == null) {
-                return null;
+                return Paranamer.EMPTY_NAMES;
             }
             if (!collector.isDebugInfoPresent()) {
-                throw new ParameterNamesNotFoundException("Parameter names not found for " + methodName);
+                if (throwExceptionIfMissing) {
+                    throw new ParameterNamesNotFoundException("Parameter names not found for " + methodName);
+                } else {
+                    return Paranamer.EMPTY_NAMES;
+                }
             }
             return collector.getResult().split(COMMA);
         }
 
-        private boolean isMethodFound() {
-            return methodFound;
-        }
-
-        private boolean isClassFound() {
-            return classFound;
-        }
     }
 
     /**
@@ -522,8 +491,13 @@ public class BytecodeReadingParanamer implements Paranamer {
                 }
                 len += n;
                 if (len == b.length) {
+                    int last = is.read();
+                    if (last < 0) {
+                        return b;
+                    }
                     byte[] c = new byte[b.length + 1000];
                     System.arraycopy(b, 0, c, 0, len);
+                    c[len++] = (byte) last;
                     b = c;
                 }
             }
@@ -584,13 +558,7 @@ public class BytecodeReadingParanamer implements Paranamer {
                 v += 6 + readInt(v + 2);
             }
 
-            // visits the class annotations
-            for (i = 1; i >= 0; --i) {
-                v = i == 0 ? ianns : anns;
-                if (v != 0) {
-                    v += 2;
-                }
-            }
+            //annotations not needed.
 
             // visits the fields
             i = readUnsignedShort(u);
@@ -607,6 +575,7 @@ public class BytecodeReadingParanamer implements Paranamer {
             i = readUnsignedShort(u);
             u += 2;
             for (; i > 0; --i) {
+                // inlined in original ASM source, now a method call
                 u = readMethod(classVisitor, c, u);
             }
 
@@ -766,33 +735,33 @@ public class BytecodeReadingParanamer implements Paranamer {
             int endIndex = index + utfLen;
             byte[] b = this.b;
             int strLen = 0;
-            int c, d, e;
+            int c;
+            int st = 0;
+            char cc = 0;
             while (index < endIndex) {
-                c = b[index++] & 0xFF;
-                switch (c >> 4) {
+                c = b[index++];
+                switch (st) {
                     case 0:
-                    case 1:
-                    case 2:
-                    case 3:
-                    case 4:
-                    case 5:
-                    case 6:
-                    case 7:
-                        // 0xxxxxxx
-                        buf[strLen++] = (char) c;
+                        c = c & 0xFF;
+                        if (c < 0x80) {  // 0xxxxxxx
+                            buf[strLen++] = (char) c;
+                        } else if (c < 0xE0 && c > 0xBF) {  // 110x xxxx 10xx xxxx
+                            cc = (char) (c & 0x1F);
+                            st = 1;
+                        } else {  // 1110 xxxx 10xx xxxx 10xx xxxx
+                            cc = (char) (c & 0x0F);
+                            st = 2;
+                        }
                         break;
-                    case 12:
-                    case 13:
-                        // 110x xxxx 10xx xxxx
-                        d = b[index++];
-                        buf[strLen++] = (char) (((c & 0x1F) << 6) | (d & 0x3F));
+
+                    case 1:  // byte 2 of 2-byte char or byte 3 of 3-byte char
+                        buf[strLen++] = (char) ((cc << 6) | (c & 0x3F));
+                        st = 0;
                         break;
-                    default:
-                        // 1110 xxxx 10xx xxxx 10xx xxxx
-                        d = b[index++];
-                        e = b[index++];
-                        buf[strLen++] = (char) (((c & 0x0F) << 12)
-                                | ((d & 0x3F) << 6) | (e & 0x3F));
+
+                    case 2:  // byte 2 of 3-byte char
+                        cc = (char) ((cc << 6) | (c & 0x3F));
+                        st = 1;
                         break;
                 }
             }
@@ -965,7 +934,7 @@ public class BytecodeReadingParanamer implements Paranamer {
          * The length of the internal name of this Java type. This field is only
          * used for reference types.
          */
-        private int len;
+        private final int len;
 
         // ------------------------------------------------------------------------
         // Constructors
