@@ -30,8 +30,6 @@
 
 package com.thoughtworks.paranamer;
 
-import com.thoughtworks.paranamer.BytecodeReadingParanamer;
-
 import java.lang.reflect.AccessibleObject;
 import java.util.WeakHashMap;
 
@@ -60,28 +58,21 @@ public class CachingParanamer implements Paranamer {
         this.delegate = delegate;
     }
 
-    /**
-     * @Deperecated Use 'new CachingParanamer(new AdaptiveParanamer())' instead.
-     */
-    public void switchtoAsm() {
-        delegate = new BytecodeReadingParanamer();
+    public String[] lookupParameterNames(AccessibleObject methodOrConstructor) {
+        return lookupParameterNames(methodOrConstructor, true);
     }
 
-    public String[] lookupParameterNames(AccessibleObject methodOrCtor) {
+    public String[] lookupParameterNames(AccessibleObject methodOrCtor, boolean throwExceptionIfMissing) {
         if(methodCache.containsKey(methodOrCtor)) {
             return (String[]) methodCache.get(methodOrCtor);
         }
 
-        String[] names = delegate.lookupParameterNames(methodOrCtor);
+        String[] names = delegate.lookupParameterNames(methodOrCtor, throwExceptionIfMissing);
         methodCache.put(methodOrCtor, names);
 
         return names;
     }
 
-    public int areParameterNamesAvailable(Class clazz, String ctorOrMethodName) {
-        return delegate.areParameterNamesAvailable(clazz, ctorOrMethodName);
-    }
-
     public String toString() {
          return new StringBuffer("[CachingParanamer delegate=")
          .append(delegate).append("]").toString();
