@@ -45,7 +45,6 @@ import java.lang.reflect.AccessibleObject;
  */
 public class DefaultParanamer implements Paranamer {
 
-    private static final String[] EMPTY_NAMES = new String[]{};
     private static final String COMMA = ",";
     private static final String SPACE = " ";
 
@@ -58,7 +57,11 @@ public class DefaultParanamer implements Paranamer {
     public DefaultParanamer() {
     }
 
-    public String[] lookupParameterNames(AccessibleObject methodOrCtor) {
+    public String[] lookupParameterNames(AccessibleObject methodOrConstructor) {
+        return lookupParameterNames(methodOrConstructor, true);
+    }
+
+    public String[] lookupParameterNames(AccessibleObject methodOrCtor, boolean throwExceptionIfMissing) {
         // Oh for some commonality between Constructor and Method !!
         Class[] types = null;
         Class declaringClass = null;
@@ -76,14 +79,17 @@ public class DefaultParanamer implements Paranamer {
         }
 
         if (types.length == 0) {
-            // faster ?
             return EMPTY_NAMES;
         }
         final String parameterTypeNames = getParameterTypeNamesCSV(types);
         final String[] names = getParameterNames(declaringClass, parameterTypeNames, name + SPACE);
         if ( names == null ){
+            if (throwExceptionIfMissing) {
             throw new ParameterNamesNotFoundException("No parameter names found for class '"+declaringClass+"', methodOrCtor " + name
                     +" and parameter types "+parameterTypeNames);
+            } else {
+                return Paranamer.EMPTY_NAMES;
+            }
         }
         return names;
     }
@@ -97,22 +103,7 @@ public class DefaultParanamer implements Paranamer {
             String parameterNames = parts[2];
             return parameterNames.split(COMMA);
         }
-        return null;
-    }
-
-    public int areParameterNamesAvailable(Class clazz, String constructorOrMethodName) {
-        String data = getParameterListResource(clazz);
-
-        if (data == null) {
-            return NO_PARAMETER_NAMES_LIST;
-        }
-
-        String line = findFirstMatchingLine(data, constructorOrMethodName + SPACE);
-        if (line.length() == 0) {
-            return NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-        }
-
-        return PARAMETER_NAMES_FOUND;
+        return Paranamer.EMPTY_NAMES;
     }
 
     private static String getParameterTypeNamesCSV(Class[] parameterTypes) {
