@@ -61,7 +61,6 @@ import java.util.zip.GZIPInputStream;
 import java.util.zip.Inflater;
 import java.util.zip.InflaterInputStream;
 import java.util.zip.ZipEntry;
-import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
 
 /**
@@ -261,43 +260,11 @@ public class JavadocParanamer implements Paranamer {
 		}
 	}
 
-	public int areParameterNamesAvailable(Class clazz,
-			String constructorOrMethodName) {
-		if ((clazz == null) || (constructorOrMethodName == null))
-			throw new NullPointerException();
+    public String[] lookupParameterNames(AccessibleObject methodOrConstructor) {
+        return lookupParameterNames(methodOrConstructor, true);
+    }
 
-		// due to general problems with this method, we just delegate
-		// the first match we find to lookupParameterNames
-		AccessibleObject accessible = null;
-		if (constructorOrMethodName.equals("<init>"))
-			accessible = clazz.getDeclaredConstructors()[0];
-		else {
-			Method[] methods = clazz.getMethods();
-			if (methods == null)
-				// this method doesn't exist
-				return NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-			for (int i = 0; i < methods.length; i++) {
-				if (methods[i].getName().equals(constructorOrMethodName)) {
-					accessible = methods[i];
-					break;
-				}
-			}
-		}
-		if (accessible == null)
-			// this method doesn't exist
-			return NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-
-		try {
-			lookupParameterNames(accessible);
-			return PARAMETER_NAMES_FOUND;
-		} catch (ParameterNamesNotFoundException e) {
-			if (e == CLASS_NOT_SUPPORTED)
-				return NO_PARAMETER_NAMES_FOR_CLASS;
-			return NO_PARAMETER_NAMES_FOR_CLASS_AND_MEMBER;
-		}
-	}
-
-	public String[] lookupParameterNames(AccessibleObject methodOrConstructor) {
+    public String[] lookupParameterNames(AccessibleObject methodOrConstructor, boolean throwExceptionIfMissing) {
 		if (methodOrConstructor == null)
 			throw new NullPointerException();
 
@@ -324,15 +291,24 @@ public class JavadocParanamer implements Paranamer {
 
 		try {
 			String[] names = getParameterNames(klass, name, types);
-			if (names == null)
-				throw new ParameterNamesNotFoundException(
-					methodOrConstructor.toString());
-			return names;
+			if (names == null) {
+                if (throwExceptionIfMissing) {
+                    throw new ParameterNamesNotFoundException(
+					    methodOrConstructor.toString());
+                } else {
+                    return Paranamer.EMPTY_NAMES;
+                }
+            }
+            return names;
 		} catch (IOException e) {
-			throw new ParameterNamesNotFoundException(
-				methodOrConstructor.toString() + " due to an I/O error: "
-						+ e.getMessage());
-		}
+            if (throwExceptionIfMissing) {
+                throw new ParameterNamesNotFoundException(
+	    			methodOrConstructor.toString() + " due to an I/O error: "
+		    				+ e.getMessage());
+            } else {
+                return Paranamer.EMPTY_NAMES;
+            }
+        }
 	}
 
 	// throws CLASS_NOT_SUPPORTED if the class file is not found in the javadocs
@@ -415,7 +391,7 @@ public class JavadocParanamer implements Paranamer {
 		Matcher matcher = pattern.matcher(javadoc);
 		if (!matcher.find())
 			// not found
-			return null;
+			return Paranamer.EMPTY_NAMES;
 
 		// found it. Lookup the parameter names.
 		String[] names = new String[types.length];
@@ -427,7 +403,7 @@ public class JavadocParanamer implements Paranamer {
 		for (int i = 0; i < types.length; i++) {
 			boolean find = matcherParams.find(start);
 			if (!find)
-				return null;
+				return Paranamer.EMPTY_NAMES;
 			start = matcherParams.end();
 			names[i] = matcherParams.group(1);
 		}
